package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
CHANGELOG

[0.2.0]

  • misc : LICENSES

    • ocaml libraries installled with skhc : GPL-3.0-or-later -> MPL-2.0
  • fix : move artist's artwork and album's cover on modify.
  • misc : always print to stderr warning (ignore quiet flag).
  • feat : parameterize artist's artwork and album's cover filename in configuration file instead of hardcoding.
  • fix : improve TRACKNUMBER metadata parsing.
  • fix : plugin's name mismatch between config section name and plugin code
  • fix : forgotten variable population.

    • BREAKING

      • artist %title : Artist name -> not populated

API CHANGES

  • Added functions

    • SkhcConfig.Plugin.init
    • SkhcConfig.Section.to_string
    • SkhcConfig.Section.pp
    • SkhcConfig.Format.artist_artwork
    • SkhcConfig.Format.album_cover
  • Added modules

    • Skhclibs.Format
  • Modified functions

    • SkhcConfig.Format.init

[0.1.0]

  • Initial release