package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
shakuhachi

shakuhachi is mainly a music collection manager but it's also a music tagger. It aims to be a rather simple collection manager extensible by plugins.

Features

  • Handle multiples music collections.
  • Organize musics in artist/album structure.
  • Create m3u playlists.
  • Set rating and playcount.
  • Edit music's metadata inside and outside your collection.
  • Replace audiofile while keeping metadata and musics' stats.
  • Write plugins or programs to manipulate your collection thanks to an Ocaml API.

Installation

shakuhachi requires Ocaml >= 4.14.0.

To build

$ opam install --deps-only .
$ dune b
# or with opam 
$ opam install shakuhachi

Usage

  • Initialization
# To create a new collection, see `skhc-init(1)`
#
# To create a new collection named <collection> and set it as the main one
$ skhc init -M -C collection -m <your-path>
  • Adding musics
# To add music to a collection, see `skhc-add(1)`
#
# To add the music `未だ.flac`
$ skhc add 未だ.flac
  • Modify music metadata
# To modify music's metadata, see `skhc-modify(1)`
#
# To add cover to And The Beat Goes On to The Whispers
$ skhc modify -P <cover-path> and the beat goes on the whispers
  • Replacing audiofile
# To replace an audiofile within a collection, see `skhc-replace(1)`
#
# To replace the music matching lisa realize, 
# keep the metadata of the old file and
# remove old file from the file system
$ skhc replace -r -M old lisa realize
replace LiSA/REALiZE/1-01 REALiZE.mp3 
path : ...
  • Tag musics
# To tag or read metadata from musics, see `skhc-tag(1)`
#
# To set album-artist to Nujabes,
# album to Modal Soul
# disc number to 1
# year to 2005
# of feather.flac eclipse.flac "modal soul.flac"
$ skhc tag -A "Nujabes" -l "Modal Soul" -d1 -y2005 feather.flac eclipse.flac "modal soul.flac"

Authors

  • erueri - creator and maintainer

License

shakuhachi is distribued under the GPL-3.0-or-later license. shakuhachi.config, shakuhachi.database, shakuhachi.libs, shakuhachi.metadata, shakuhachi.plugin, and shakuhachi.query are distribued under the MPL-2.0 license.