package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module SkhcConfig.MiscSource

Operations on the .misc section.

Sourcetype t

type of the misc section.

Sourceval section : string

section name.

Sourceval default : t

default is the section used if the default section is missing from the configuration file.

Sourceval init : unicode_normalization_form:Uunf.form -> t

init ~unicode_normalization_form creates a misc section with unicode_normalization_form.

Sourceval of_section : Section.t -> t

of_section section parse section to create a misc section. If an expected key is missing from section, the key is associated with its default value.

Sourceval to_section : t -> Section.t

to_section default serializes misc into a section.

Sourceval unicode_normalization_form : t -> Uunf.form

unicode_normalization_form default returns the unicode normalization form used.