find ~compilation database artists album tries to find an album in database with the name album.name and at least an arist with the name is in artists unless compilation, in this case only the album.name is check to discriminate albums
find_or_insert ?insert_on_found ?roles ~compilation database artists album tries to find an album in database with the name album.name and at least an artist with the name is in artists unless compilation, in this case only the album.name is check to discriminate albums. If no album is found, find_or_insert will try to insert album to database.
If insert_on_found, find_or_insert will try to add artists to the already existing album.
If roles, artists is filtered to only use artist having a role within roles. The unmodified artists list is used afterward.
before0.3.2,
no insert_on_found optional argument and the behavior of find_or_insert was equivalent to insert_on_found = false.