package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Field.FloatSource

Operations on float field node.

CAVEATS: Despite being in the query ast, float field are not currently supported by the parser. While it's possible to build a query containing float field using the ast, it's not possible to build one from positional nor s-expression query.

Sourcetype t =
  1. | AvgDuration of float Range.t
  2. | AvgRating of float Range.t
  3. | AvgSize of float Range.t

type of float field node

Sourceval avg_duration : float Range.t -> t

avg_duration r is AvgDuration r.

Sourceval avg_rating : float Range.t -> t

avg_rating r is AvgRating r.

Sourceval avg_size : float Range.t -> t

avg_size r is AvgSize r.

Sourceval to_string : t -> string

to_string field returns a string representation of field.

Sourceval to_sexp : t -> string

to_sexp field is to_string.