package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module SkhcQuery.SubstringSource

Operations on substring node' values

Sourcetype t =
  1. | STitle
  2. | SSortName
  3. | SArtist
  4. | SAlbum
  5. | SAlbumArtist
  6. | SComposer
  7. | SGenre

type of substring node' values

Sourceval assocs : (string * t) list

assocs is each substring associated with their key.

Sourceval to_string : t -> string

to_string substring is a Keys associated with substring.

Sourceval of_string : string -> t option

of_string s tries to parse s a key associated with a substring.

Sourceval default : t list

default list of substring used by skhc when the query.field key in configuration file is missing.

Sourceval compare : t -> t -> int

compare s0 s1 is Stdlib.compare s0 s1.