package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file misc.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
(**********************************************************************************************)
(*                                                                                            *)
(* This file is part of shakuhachi: a music collection manager                                *)
(* Copyright (C) 2025 Yves Ndiaye                                                             *)
(*                                                                                            *)
(* shakuhachi is free software: you can redistribute it and/or modify it under the terms      *)
(* of the GNU General Public License as published by the Free Software Foundation,            *)
(* either version 3 of the License, or (at your option) any later version.                    *)
(*                                                                                            *)
(* shakuhachi is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;    *)
(* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           *)
(* PURPOSE.  See the GNU General Public License for more details.                             *)
(* You should have received a copy of the GNU General Public License along with shakuhachi.   *)
(* If not, see <http://www.gnu.org/licenses/>.                                                *)
(*                                                                                            *)
(**********************************************************************************************)

module Keys = struct
  let unicode_nf = "unicode_normalization_form"
end

module Default = struct
  let unicode_normalization_form : Uunf.form = `NFC
end

module Unicode = struct
  let form_of_string = function
    | "nfd" ->
        `NFD
    | "nfkd" ->
        `NFKD
    | "nfkc" ->
        `NFKC
    | "nfc" ->
        `NFC
    | _ ->
        `NFC

  let form_to_string = function
    | `NFC ->
        "nfc"
    | `NFD ->
        "nfd"
    | `NFKD ->
        "nfkd"
    | `NFKC ->
        "nfkc"
end

type t = { unicode_normalization_form : Uunf.form }

let section = "misc"
let default : t = { unicode_normalization_form = `NFC }
let init ~unicode_normalization_form = { unicode_normalization_form }

(*let conv ~default key section =
  match Section.assoc key section with Some e -> e | None -> default*)

let of_section section =
  let unicode_normalization_form =
    section
    |> Section.assoc Keys.unicode_nf
    |> Option.map Unicode.form_of_string
    |> Option.value ~default:Default.unicode_normalization_form
  in
  { unicode_normalization_form }

let to_section t =
  let unicode = Unicode.form_to_string t.unicode_normalization_form in
  Section.init section [ (Keys.unicode_nf, unicode) ]

let unicode_normalization_form t = t.unicode_normalization_form