package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file plugin.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
(**********************************************************************************************)
(*                                                                                            *)
(* This file is part of shakuhachi: a music collection manager                                *)
(* Copyright (C) 2025 Yves Ndiaye                                                             *)
(*                                                                                            *)
(* shakuhachi is free software: you can redistribute it and/or modify it under the terms      *)
(* of the GNU General Public License as published by the Free Software Foundation,            *)
(* either version 3 of the License, or (at your option) any later version.                    *)
(*                                                                                            *)
(* shakuhachi is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;    *)
(* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           *)
(* PURPOSE.  See the GNU General Public License for more details.                             *)
(* You should have received a copy of the GNU General Public License along with shakuhachi.   *)
(* If not, see <http://www.gnu.org/licenses/>.                                                *)
(*                                                                                            *)
(**********************************************************************************************)

module Config = struct
  include Map.Make (String)

  type nonrec t = string t
end

type t = string * Config.t

let prefix = "plugins."

let of_section (section : Section.t) =
  let section_name = Section.name section in
  let plugin_name =
    match String.split_on_char '.' section_name with
    (* Empty string since section name start with a dot*)
    | "" :: "plugins" :: q ->
        String.concat "." q
    | _ ->
        (* should append *) section_name
  in
  let values = Section.values section in
  let values = Config.of_seq (List.to_seq values) in
  (plugin_name, values)

let section_matches section = String.starts_with ~prefix section
let name t = fst t
let config t = snd t

let to_section t =
  let name, values = t in
  let values = Config.bindings values in
  let section_name = prefix ^ name in
  Section.init section_name values