package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file query.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
(**********************************************************************************************)
(*                                                                                            *)
(* This file is part of shakuhachi: a music collection manager                                *)
(* Copyright (C) 2025 Yves Ndiaye                                                             *)
(*                                                                                            *)
(* shakuhachi is free software: you can redistribute it and/or modify it under the terms      *)
(* of the GNU General Public License as published by the Free Software Foundation,            *)
(* either version 3 of the License, or (at your option) any later version.                    *)
(*                                                                                            *)
(* shakuhachi is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;    *)
(* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           *)
(* PURPOSE.  See the GNU General Public License for more details.                             *)
(* You should have received a copy of the GNU General Public License along with shakuhachi.   *)
(* If not, see <http://www.gnu.org/licenses/>.                                                *)
(*                                                                                            *)
(**********************************************************************************************)

module Keys = struct
  let fields = "fields"
end

type t = { fields : SkhcQuery.Substring.t list }

let section = "query"
let default = { fields = SkhcQuery.Substring.default }
let init fields = { fields }

let of_section section =
  let fields =
    match Section.assoc Keys.fields section with
    | None ->
        SkhcQuery.Substring.default
    | Some s ->
        s |> String.split_on_char ' '
        |> List.filter_map SkhcQuery.Substring.of_string
  in
  { fields }

let to_section t =
  let { fields } = t in
  let fields =
    String.concat " " (List.map SkhcQuery.Substring.to_string fields)
  in
  Section.init section [ (Keys.fields, fields) ]

let fields t = t.fields