package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file util.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
(**********************************************************************************************)
(*                                                                                            *)
(* This file is part of shakuhachi: a music collection manager                                *)
(* Copyright (C) 2025 Yves Ndiaye                                                             *)
(*                                                                                            *)
(* shakuhachi is free software: you can redistribute it and/or modify it under the terms      *)
(* of the GNU General Public License as published by the Free Software Foundation,            *)
(* either version 3 of the License, or (at your option) any later version.                    *)
(*                                                                                            *)
(* shakuhachi is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;    *)
(* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           *)
(* PURPOSE.  See the GNU General Public License for more details.                             *)
(* You should have received a copy of the GNU General Public License along with shakuhachi.   *)
(* If not, see <http://www.gnu.org/licenses/>.                                                *)
(*                                                                                            *)
(**********************************************************************************************)

module StringUtil = struct
  let make_path_safe s = String.map (function '/' | '\000' -> '_' | c -> c) s
end

module ListUtil = struct
  let rec fold_ok f acc = function
    | [] ->
        Result.ok acc
    | t :: q ->
        let ( let* ) = Result.bind in
        let* acc = f acc t in
        fold_ok f acc q
end

module FileSys = struct
  let split_path s =
    let regex = Re.(compile @@ str Filename.dir_sep) in
    Re.split regex s

  let rec remove_prefix prefix path =
    match (prefix, path) with
    | [], [] | _ :: _, [] ->
        String.empty
    | [], (_ :: _ as s) ->
        String.concat Filename.dir_sep s
    | t1 :: q1, (t2 :: q2 as s) ->
        if String.equal t1 t2 then
          remove_prefix q1 q2
        else
          String.concat Filename.dir_sep s

  let remove_prefix prefix path =
    let prefix = Unix.realpath prefix in
    let path = Unix.realpath path in
    let prefix = split_path prefix in
    let path = split_path path in
    remove_prefix prefix path

  let read_chan chan =
    let buffer = Buffer.create 4_096 in
    let size = 1024 in
    let bytes = Bytes.create size in
    let continue = ref true in

    let () =
      while !continue do
        let read = input chan bytes 0 size in
        let () = Buffer.add_subbytes buffer bytes 0 read in
        continue := read = size
      done
    in
    Buffer.contents buffer
end