package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file event.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
(**********************************************************************************************)
(*                                                                                            *)
(* This file is part of shakuhachi: a music collection manager                                *)
(* Copyright (C) 2025 Yves Ndiaye                                                             *)
(*                                                                                            *)
(* shakuhachi is free software: you can redistribute it and/or modify it under the terms      *)
(* of the GNU General Public License as published by the Free Software Foundation,            *)
(* either version 3 of the License, or (at your option) any later version.                    *)
(*                                                                                            *)
(* shakuhachi is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;    *)
(* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           *)
(* PURPOSE.  See the GNU General Public License for more details.                             *)
(* You should have received a copy of the GNU General Public License along with shakuhachi.   *)
(* If not, see <http://www.gnu.org/licenses/>.                                                *)
(*                                                                                            *)
(**********************************************************************************************)

(** Module on shakuhachi plugin event. *)

(** type of shakuhachi event *)
type t =
  | OnLoaded
  | OnScanFile
  | OnScanMetadata
  | OnFileMoved
  | OnMusicModified
  | OnArtistModified
  | OnAlbumModified
  | OnPlaylistModified
  | OnPlaylistTracksUpdated

(** [to_string e] is a string representation of the event. *)
let to_string = function
  | OnLoaded ->
      "on_loaded"
  | OnScanFile ->
      "on_scan_file"
  | OnScanMetadata ->
      "on_scan_metadata"
  | OnFileMoved ->
      "on_file_moved"
  | OnMusicModified ->
      "on_music_modified"
  | OnArtistModified ->
      "on_artist_modified"
  | OnAlbumModified ->
      "on_album_modified"
  | OnPlaylistModified ->
      "on_playlist_modified"
  | OnPlaylistTracksUpdated ->
      "on_playlist_tracks_updated"