package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file tagger.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
(*****************************************************************************)
(*                                                                           *)
(*  Copyright (C) 2026 Yves Ndiaye                                           *)
(*                                                                           *)
(* This Source Code Form is subject to the terms of the Mozilla Public       *)
(* License, v. 2.0. If a copy of the MPL was not distributed with this       *)
(* file, You can obtain one at https://mozilla.org/MPL/2.0/.                 *)
(*                                                                           *)
(*****************************************************************************)

(** Operations for the tagger plugin.

    @since 0.3.0 *)

(** Signature of a shakuhachi tagger plugin *)
module type S = sig
  val tag :
    ( string * SkhcMetadata.Metadata.t -> SkhcMetadata.Metadata.t,
      (string * SkhcMetadata.Metadata.t) list -> SkhcMetadata.Metadata.t list
    )
    Either.t
  (** [tag f] is the function called by {i skhc-tag(1)} if the plugin is given.
      - If [f = Either.Left fun]:
      - [fun] is called once on each file passed to {i skhc-tag(1)}. In
        particular, if {i n} audio files are passed to {i skhc-tag(1)}, [fun]
        will be called [n] times.
      - If [f = Either.Right fun]:
      - [fun] is called once and takes all at once the audio files passed to
        {i skhc-tag(1)}. [fun] must return the same number of metatdata as
        given. Otherwise, no tagging will occurs. *)
end

(* An hack, another one to force the linkage of Either. *)
let _ = Either.left
let plugin : (module S) option ref = ref None

(** [register plugin] registers [plugin] to be loaded by [skhc-tag]. *)
let register p = plugin := Some p

(** [plugins ()] returns th plugins currently registered. *)
let plugin () = !plugin