package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file keys.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
(**********************************************************************************************)
(*                                                                                            *)
(* This file is part of shakuhachi: a music collection manager                                *)
(* Copyright (C) 2025 Yves Ndiaye                                                             *)
(*                                                                                            *)
(* shakuhachi is free software: you can redistribute it and/or modify it under the terms      *)
(* of the GNU General Public License as published by the Free Software Foundation,            *)
(* either version 3 of the License, or (at your option) any later version.                    *)
(*                                                                                            *)
(* shakuhachi is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;    *)
(* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR           *)
(* PURPOSE.  See the GNU General Public License for more details.                             *)
(* You should have received a copy of the GNU General Public License along with shakuhachi.   *)
(* If not, see <http://www.gnu.org/licenses/>.                                                *)
(*                                                                                            *)
(**********************************************************************************************)

(** Operation on keys for field node. *)

let id = "%id"
let title = "%title"
let sort_name = "%sort_name"
let artist = "%artist"
let album = "%album"
let album_artist = "%album_artist"
let composer = "%composer"
let genre = "%genre"
let track = "%track"
let year = "%year"
let disc = "%disc"
let plays = "%plays"
let size = "%size"
let rating = "%rating"
let art = "%art"
let flag = "%flag"
let album_count = "%album_count"
let avg_duration = "%avg_duration"
let avg_rating = "%avg_rating"
let avg_size = "%avg_size"
let length = "%length"
let path = "%path"
let extension = "%ext"