package shakuhachi

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module SkhcDb.AlbumParticipantSource

Operations on album's participant

Sourcetype t

type of album' participants

Sourceval init : album_id:int64 -> artist_id:int64 -> Role.t -> t

init ~album_id ~artist_id role creates a new relation by associated a music with album_id an artist with artist_id with a role.

Sourceval equal : t -> t -> bool

equal a1 a2 tests whether a1 equals a2. All fields are used for the test.

Sourceval compare : t -> t -> int

compare a1 a2 is Stdlib.compare.

Sourceval insert : Database.t -> t -> (unit, Sqlite3.Rc.t * string) result

insert db participant inserts participant inside db.

Sourceval update_artist : Database.t -> int64 -> int64 -> (unit, Sqlite3.Rc.t * string) result

update_artist db old_id new_id replaces old_id by new_id.

Sourceval find_or_insert : compilation:bool -> Database.t -> (Artist.t * Role.t) list -> Album.t -> (Album.t, Sqlite3.Rc.t * string) result

find_or_insert ~compilation database artists album tries to find an album in database with the name name album and at least an artist with the name is in artists unless compilation, in this case only the name album is checked to discriminate albums